/*******************************************************************************
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 *******************************************************************************/
package org.ofbiz.minilang.method.entityops;

import org.w3c.dom.*;

import org.ofbiz.minilang.*;
import org.ofbiz.minilang.method.*;

/**
 * Clears all Entity Engine Caches
 */
public class ClearEntityCaches extends MethodOperation {
    public static final class ClearEntityCachesFactory implements Factory<ClearEntityCaches> {
        public ClearEntityCaches createMethodOperation(Element element, SimpleMethod simpleMethod) {
            return new ClearEntityCaches(element, simpleMethod);
        }

        public String getName() {
            return "clear-entity-caches";
        }
    }

    public ClearEntityCaches(Element element, SimpleMethod simpleMethod) {
        super(element, simpleMethod);
    }

    @Override
    public boolean exec(MethodContext methodContext) {
        methodContext.getDelegator().clearAllCaches();
        return true;
    }

    @Override
    public String rawString() {
        // TODO: something more than the empty tag
        return "<clear-entity-caches/>";
    }
    @Override
    public String expandedString(MethodContext methodContext) {
        // TODO: something more than a stub/dummy
        return this.rawString();
    }
}
